@@ -1,7 +1,7 @@
 /*
  * /etc/network/interfaces parser
  *
- * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
+ * Copyright (C) 2003--2010  Enrico Zini <enrico@debian.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 #include "scanner/script.h"
 #include "scanner/dhcp.h"
 #include "scanner/linkbeat.h"
-#include "Environment.h"
+#include "options.h"
 
 #include <map>
 #include <memory>
@@ -70,6 +70,153 @@ public:
 	}
 };
 
+struct IfupdownFilter : public IfaceFilter
+{
+	set<string> ifupdownProfiles;
+	bool ifupdownProfilesMatchInverted;
+
+	IfupdownFilter() : ifupdownProfilesMatchInverted(false)
+	{
+	}
+	virtual ~IfupdownFilter() {}
+
+	virtual bool operator()(const std::string& name) const
+	{
+		bool hasmatch = false;
+		if (options.autofilter) 
+			hasmatch = (name.substr(0, options.iface.size()+1) == options.iface+"-") ;
+		else 
+			hasmatch = ifupdownProfilesMatchInverted || (ifupdownProfiles.size() == 0);
+
+		if (ifupdownProfilesMatchInverted && ifupdownProfiles.find(name) != ifupdownProfiles.end())
+		{
+			hasmatch = false;
+		}
+		else if (!ifupdownProfilesMatchInverted && ifupdownProfiles.find(name) != ifupdownProfiles.end())
+		{
+			hasmatch = true;
+		} 
+		return hasmatch;
+	}
+};
+
+void IfaceParser::parseMapping(FILE* in)
+{
+	auto_ptr<IfupdownFilter> filter(new IfupdownFilter);
+
+	::debug("program name is guessnet-ifupdown: enabling ifupdown mode\n");
+
+	// Read stuff from stdin
+	wibble::ERegexp null_line("^[[:blank:]]*(#.*)?$");
+	wibble::ERegexp parm_line("^[[:blank:]]*([A-Za-z_-]+):[[:blank:]]*(.+)$", 3);
+
+	string line;
+	int linenum = 1;
+	int c;
+	while ((c = fgetc(in)) != EOF)
+	{
+		if (c != '\n')
+			line += c;
+		else
+		{
+			if (null_line.match(line))
+			{
+				//fprintf(stderr, "EMPTY\n");
+			}
+			else if (parm_line.match(line))
+			{
+				string name = parm_line[1];
+				Tokenizer t(parm_line[2]);
+				string val = t.next();
+
+				if (name == "default")
+				{
+					if (!val.empty())
+						options.defprof = val;
+				}
+				else if (name == "verbose")
+				{
+					if (!val.empty())
+						util::Output::get().verbose(val == "true");
+				}
+				else if (name == "debug")
+				{
+					if (!val.empty())
+						util::Output::get().debug(val == "true");
+				}
+				else if (name == "syslog")
+				{
+					if (!val.empty())
+						util::Output::get().syslog(val == "true");
+				}
+				else if (name == "timeout")
+				{
+					int v = atoi(val.c_str());
+					if (v > 0)
+						options.timeout = v;
+				}
+				else if (name == "init-time")
+				{
+					int v = atoi(val.c_str());
+					if (v > 0)
+						options.init_timeout = v;
+				}
+				else if (name == "autofilter")
+				{
+					options.autofilter = val == "true";
+				}
+				else if (name == "init-delay")
+				{
+					int v = atoi(val.c_str());
+					if (v > 0)
+						options.initdelay = v;
+				}
+				else if (name == "iwscan-tries")
+				{
+					int v = atoi(val.c_str());
+					if (v > 0)
+						options.iwscan_tries = v;
+				}
+			}
+			else
+			{
+				Tokenizer t(line);
+				bool first = true;
+				for (string w = t.next(); !w.empty(); w = t.next())
+				{
+					if (first)
+					{
+						filter->ifupdownProfilesMatchInverted = w[0] == '!';
+						first = false;
+					}
+					if (w[0] == '!')
+					{
+						if (!filter->ifupdownProfilesMatchInverted)
+							throw wibble::exception::Consistency(
+									"parsing list of interfaces to use",
+									"found negated interface "+w+" after a normal interface");
+						filter->ifupdownProfiles.insert(w.substr(1));
+						std::string dm = "Added " + w + "\n";
+						::debug(dm.c_str());
+					} else {
+						if (filter->ifupdownProfilesMatchInverted)
+							throw wibble::exception::Consistency(
+									"parsing list of interfaces to use",
+									"found normal interface "+w+" after negated interface");
+						filter->ifupdownProfiles.insert(w);
+						std::string dm = "Added " + w + "\n";
+						::debug(dm.c_str());
+					}
+				}
+			}
+			line = string();
+			linenum++;
+		}
+	}
+
+	options.iface_filter = filter.release();
+}
+
 /* Parse the input from `input'
  * To make it simple, use regexps on input lines instead of implementing a real
  * parser.
@@ -243,7 +390,7 @@ void IfaceParser::parse(FILE* input, const IfaceFilter& filter)
 			{
 				if (profileName.size())
 				{
-					ScanBag::get().add(scanner::Script::createScan(profileName, string("pppoe -I ") + Environment::get().iface() + " -A >/dev/null 2>&1"));
+					ScanBag::get().add(scanner::Script::createScan(profileName, string("pppoe -I ") + options.iface + " -A >/dev/null 2>&1"));
 					found++;
 				}
 			}
@@ -354,7 +501,7 @@ void IfaceParser::parse(FILE* input, const IfaceFilter& filter)
 				{
 					//debug("Will use tag %.*s as default\n", PFSTR(profileName));
 
-					Environment::get().defprof(profileName);
+					options.defprof = profileName;
 				}
 			}
 			else if (generic_guessnet_line.match(line))
